/*
 * Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
 *
 * This file is part of NRAO Workspaces.
 *
 * Workspaces is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Workspaces is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
 */
import {NgModule} from '@angular/core';
import {Routes, RouterModule} from '@angular/router';

import {ArchiveSearchComponent} from "./archive-search.component";
import {ObservationResultsComponent} from "./observation-results/observation-results.component";
import {ImageResultsComponent} from "./image-results/image-results.component";
import {ProjectResultsComponent} from "./project-results/project-results.component";

import {ObservationDetailsComponent} from "./observation-details/observation-details.component";
import {ProjectDetailsComponent} from "./project-details/project-details.component";

const routes: Routes = [
  {
    path: 'Search', component: ArchiveSearchComponent, children: [
      {path: 'Observations', component: ObservationResultsComponent},
      {path: 'Images', component: ImageResultsComponent},
      {path: 'Projects', component: ProjectResultsComponent},
      {path: '', redirectTo: 'Observations', pathMatch: 'full'},
      {path: '**', redirectTo: 'Observations', pathMatch: 'full'}
    ]
  },
  {path: 'Observation/:objectId', component: ObservationDetailsComponent},
  {path: 'Project/:objectId', component: ProjectDetailsComponent},
  {path: '', redirectTo: 'Search', pathMatch: 'full'},
  {path: '**', redirectTo: 'Search', pathMatch: 'full'}
];

@NgModule({
  imports: [RouterModule.forChild(routes)],
  exports: [RouterModule]
})
export class ArchiveSearchRoutingModule {
}
