# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""update download workflow shell script to use productfetcher

Revision ID: 23e9b7724ad8
Revises: 2682b801de33
Create Date: 2021-05-17 16:07:09.199519

"""
import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "23e9b7724ad8"
down_revision = "2682b801de33"
branch_labels = None
depends_on = None


def upgrade():
    shell_script_content = """#!/bin/sh
./productfetcher --product-locator $1
./deliver -r . --prefix $2
"""

    op.execute(
        "UPDATE workflow_templates " f"SET content = E'{shell_script_content}' " "WHERE filename = 'test_download.sh'"
    )

    submit_file_content = """executable = test_download.sh
arguments = {{product_locator}} {{request_id}}

output = test_download.out
error = test_download.err
log = test_download.log

SBIN_PATH = /lustre/aoc/cluster/pipeline/$ENV(CAPO_PROFILE)/workspaces/sbin
should_transfer_files = yes
transfer_input_files = $(SBIN_PATH)/productfetcher, $(SBIN_PATH)/deliver

getenv = True

queue
"""

    op.execute(
        "UPDATE workflow_templates "
        f"SET content = E'{submit_file_content}' "
        "WHERE filename = 'test_download.condor'"
    )


def downgrade():
    shell_script_content = """#!/bin/sh
./datafetcher --product-locator $1
./deliver -r . --prefix $2
"""

    op.execute(
        "UPDATE workflow_templates " f"SET content = E'{shell_script_content}' " "WHERE filename = 'test_download.sh'"
    )

    submit_file_content = """executable = test_download.sh
arguments = {{product_locator}} {{request_id}}

output = test_download.out
error = test_download.err
log = test_download.log

SBIN_PATH = /lustre/aoc/cluster/pipeline/$ENV(CAPO_PROFILE)/workspaces/sbin
should_transfer_files = yes
transfer_input_files = $(SBIN_PATH)/datafetcher, $(SBIN_PATH)/deliver

queue
"""

    op.execute(
        "UPDATE workflow_templates "
        f"SET content = E'{submit_file_content}' "
        "WHERE filename = 'test_download.condor'"
    )
