# Future Product Architecture

When I request data from the archive, the archive goes and retrieves it. What about if I want to retrieve the data, then
calibrate it, then split the data into several pieces and image each piece? Each of those requests is an independent _
capability request_, but how do we arrange for them to occur without making the user wait for the first request to
complete before doing the second, and the second and then the third?

How can the DAs set up a calibration pipeline, so that whenever a new piece of data is archived, it triggers a
calibration capability? How can a user arrange for their custom processing steps to be executed as soon as their data is
ingested into the archive?

The answer to these questions is the future product system.

## What is a product?

To the archive, a product is something with a _science product locator_ in the archive. Science product locators are
URIs and look like this: `uid://alma/execblock/39113b0e-2f2e-4a73-8c95-c75c18b98ee9`.

Every product locator string contains the instrument that generated the product, the "type" of the product that was
generated (`execblock` and `calibration` are common possibilities) and then a globally unique identifier (GUID). If you
have this locator, you have the master key for the archive for this product and can access it and cause it to be
downloaded.

It is **extremely important** that you not try to parse the product locator. They to be treated as opaque identifiers,
except to the product locator service in the archive.

## What is a future product?

The future product system is a way of referring to products that already exist as well as products that do not yet
exist. Future products exist in one of two states: resolved or not-yet-resolved. The "base case" of the type of future
products wraps a science product locator and is resolved. There is a second "base case": An external execution block
identifier, which is resolved once that execution block is ingested and the science product locator is obtained. Finally
there is an inductive type: a capability request, which is resolved once the request is complete.

The capability-request future product is what allows the creation of follow-on requests. Each time a follow-on request
is generated, it is generated against a future product of the original request. If this is done a second time, you get a
follow-on for a follow-on request. In this way you can construct a calibration which leads to some kind of data split,
which leads in a tree-like fashion to a number of imaging requests.

If the original request is for an external execution block identifier, then you have the beginning of an integrated
calibration pipeline system, where the system is expecting an execution block to be ingested, and expecting to calibrate
it once that happens. The user ought to then be able to base (say) a custom imaging request as a follow-on for the
standard calibration, and not have to do anything reactively as the execblock or calibration become available in the
archive.

## How does it work?

When a future product is defined, it begins life in either the resolved or unresolved states, depending on whether it is
wrapping a product locator (resolved) or one of the other types (unresolved).

A new system exists in the capability service, the future product system, which keeps track of future products. Each
product is tracked by its original definition and its current state of resolution. As new requests enter the system, if
they reuse a definition, they become linked because they share the original definition.

The future product system is receiving events from the archive as products are ingested, and events from the capability
system as requests reach completion. It matches events from these sources to future products that are awaiting events.

At the moment a future product becomes resolved, an event is generated by the future product system which is broadcast
to anyone that might be interested. The capability system is generally going to be interested in these events, so when
they come in, the capability service will see if it has any capability requests sitting at the AWAIT PRODUCT stage,
waiting on this future product. If there is an AWAIT PRODUCT for this product, that step is now completed and the
capability step sequence advances to the next step.

## Examples


### Existing product
1. A capability request is made using a science product locator. The future product wraps the science product locator and it starts life resolved.

3. The AWAIT PRODUCT step sees that the product is already resolved, and the engine advances to the
next step in the sequence immediately.

### Follow-on request

1. A capability request is made as a follow on against the previous request. The future product is a capability-request
future product; it is defined by wrapping the capability request identifier and begins life unresolved.

2. The AWAIT PRODUCT step notices that the future product is unresolved and leaves the request in a dormant state.

3. When the previous request completes, the future product system receives the event from the capability system that
   a capability request completed.

4. The future product system matches the request-complete event to the future product that was awaiting it. The
current-state-of-resolution is updated to a reference to that capability request's result. There are no more resolution
steps required, so the future product is now resolved.

5. The future product system emits an event that this future product is resolved and here is the resolution.

6. The capability system receives that event and matches it to the request in the AWAIT PRODUCT state. The step is
   now complete and the engine advances the request to the next step.

### External execution block identifier

1. A capability request is made against an external execution block identifier. The future product is an
external-execution-block-id future product; it begins life unresolved.

2. The AWAIT PRODUCT step notices that it is unresolved and leaves the request in a dormant state.

3. When the execution block is ingested, an event enters the future product system with the science product locator.
   The future product is located and the current-state-of-resolution is advanced to this science product locator,
   and the product becomes resolved.

4. The future product system emits an event with this resolution.

5. The capability system receives the event and matches to the request in the AWAIT PRODUCT state. The step is now
   complete and the engine advances the request to the next step.


## Notes from call with Nathan

- Capability Request could have type Filesystem -> Filesystem
- Future Product system could become Product system, adopt responsibility for running the data fetcher or doing copy
  from earlier request to follow-on request
- How do we deal with capabilities that do not need files retrieved? They don't have AWAIT PRODUCT at the outset
- How do we deal with capabilities that ingest or are otherwise final? They still have a working directory at their
  conclusion which can be used as an input to the next request
- TODO How do we handle requests that need to fetch multiple things before executing? Restores, for instance,
  need to retrieve the data AND retrieve the calibration tables? Open question.
   - Multiple AWAIT PRODUCTs? Or Product System is parameterized by a list of products?
   - What about follow-ons that also need to request data? Think CIPL -> image
